/****************************************************************************/
/*  Test MOD-24LR / nrf24L01                                                */
/****************************************************************************/
/*  LEnsE / Julien VILLEMEJANE       /   Institut d'Optique Graduate School */
/****************************************************************************/
/*  Brochage                                                                */
/*      TO COMPLETE                                                         */
/****************************************************************************/
/*  Test réalisé sur Nucléo-F411RE                                          */
/****************************************************************************/

#include "mbed.h"
#include "MOD24_NRF.h"
#include "nRF24.h"

Serial          debug_pc(USBTX, USBRX);
//InterruptIn     bp_int(USER_BUTTON);

#include "TEMPHUM_14_CLICK.h"


#define WAIT_TIME_MS 500 

/// Create an I2C interface with specific pins : SDA / SCL
I2C                 my_i2c(D14, D10); // SDA / SCL

/// Create a digital output for reset input of the sensor
DigitalOut          my_rst(D9);
/// Create a Color_10_Click module connection with an I2C interface and a Digital Out for the led
TempHum_14_Click    my_sensor(&my_i2c, &my_rst);


//moteur
PwmOut moteur_2_avancer(PB_14);
PwmOut moteur_2_reculer(PB_13);
PwmOut moteur_3_avancer(PC_9);
PwmOut moteur_3_reculer(PB_8);

DigitalIn codeur_1_M3(PB_10);
DigitalIn codeur_1_M2(PB_1);

//DigitalIn mybutton(PC_13);


void Avancer_ou_reculer(int D, double V) 
// D une distance algébrique
// V la vitesse voulue
{   
    // init paramètre
    int old_codeur31, new_codeur31;
    int old_codeur21, new_codeur21;
    int compteur_codeur31;
    int compteur_codeur21;
    double N;
    double rc;
  
    compteur_codeur31=0;
    compteur_codeur21=0;
    old_codeur31 = codeur_1_M3;
    old_codeur21 = codeur_1_M2;

    moteur_2_avancer.write(0);
    moteur_2_reculer.write(0);
    moteur_3_avancer.write(0);
    moteur_3_reculer.write(0);

    rc=V;
    N=floor(D*2.78); // facteur convertissant la distance a parcourir en nombre de rotation de la roue

    if (D>0){ // si positif on avance

        moteur_2_avancer.write(rc);
        moteur_3_avancer.write(rc);

        // des que une des roues effectue le bon nombre de rotation on arrete
        while( (compteur_codeur21 < N) && (compteur_codeur31 < N) ){

            new_codeur31 = codeur_1_M3;
            new_codeur21 = codeur_1_M2;

            // comptage de tours
            if (new_codeur31 != old_codeur31) {
            
            compteur_codeur31 = compteur_codeur31 + 1;
        }
        old_codeur31=new_codeur31;

            // comptage de tours
            if (new_codeur21 != old_codeur21) {
            compteur_codeur21 = compteur_codeur21 + 1;
        }
        old_codeur21=new_codeur21;

        //printf("CompteurM3 = %d  \r\n", compteur_codeur31);
        //printf("CompteurM2 = %d  \r\n", compteur_codeur21);
    
    }

    moteur_2_avancer.write(0);
    moteur_2_reculer.write(0);
    moteur_3_avancer.write(0);
    moteur_3_reculer.write(0);

    }

    if (D<0){ // si negatif on recule en fait la mm chose
        
        N=-N; // prend la valeur positive
        moteur_2_reculer.write(rc);
        moteur_3_reculer.write(rc);

        while( (compteur_codeur21 < N) && (compteur_codeur31 < N) ){

            new_codeur31 = codeur_1_M3;
            new_codeur21 = codeur_1_M2;
            
            if (new_codeur31 != old_codeur31) {
            compteur_codeur31 = compteur_codeur31 + 1;
            //printf("je recule \r\n");
        }
        old_codeur31=new_codeur31;
            if (new_codeur21 != old_codeur21) {
            compteur_codeur21 = compteur_codeur21 + 1;
        }
        old_codeur21=new_codeur21;

        //printf("CompteurM3 = %d  \r\n", compteur_codeur31);
        //printf("CompteurM2 = %d  \r\n", compteur_codeur21);

    
    }
    moteur_2_avancer.write(0);
    moteur_2_reculer.write(0);
    moteur_3_avancer.write(0);
    moteur_3_reculer.write(0);    
    }
}


void Tourner_angle(int D, double V) 
// D un angle
// V la vitesse voulue
{   
    int old_codeur31, new_codeur31;
    int old_codeur21, new_codeur21;
    int compteur_codeur31;
    int compteur_codeur21;
    double N;
    double rc;
  
    compteur_codeur31=0;
    compteur_codeur21=0;
    old_codeur31 = codeur_1_M3;
    old_codeur21 = codeur_1_M2;

    moteur_2_avancer.write(0);
    moteur_2_reculer.write(0);
    moteur_3_avancer.write(0);
    moteur_3_reculer.write(0);

    rc=V;
    N=floor(D*2.78); // facteur convertissant l'angle a parcourir en nombre de rotation de la roue

    // le programme est similaire au précédent en tournant les roues de manière opposées
    if (D>0){

        moteur_2_reculer.write(rc);
        moteur_3_avancer.write(rc);

        while( (compteur_codeur21 < N) && (compteur_codeur31 < N) ){

            new_codeur31 = codeur_1_M3;
            new_codeur21 = codeur_1_M2;

            if (new_codeur31 != old_codeur31) {
            //printf("j'avance \r\n");
            compteur_codeur31 = compteur_codeur31 + 1;
        }
        old_codeur31=new_codeur31;

            if (new_codeur21 != old_codeur21) {
            compteur_codeur21 = compteur_codeur21 + 1;
        }
        old_codeur21=new_codeur21;

        //printf("CompteurM3 = %d  \r\n", compteur_codeur31);
        //printf("CompteurM2 = %d  \r\n", compteur_codeur21);
    
    }

    moteur_2_avancer.write(0);
    moteur_2_reculer.write(0);
    moteur_3_avancer.write(0);
    moteur_3_reculer.write(0);

    }

    if (D<0){
        
        N=-N;
        moteur_2_avancer.write(rc);
        moteur_3_reculer.write(rc);

        while( (compteur_codeur21 < N) && (compteur_codeur31 < N) ){

            new_codeur31 = codeur_1_M3;
            new_codeur21 = codeur_1_M2;
            
            if (new_codeur31 != old_codeur31) {
            compteur_codeur31 = compteur_codeur31 + 1;
            //printf("je recule \r\n");
        }
        old_codeur31=new_codeur31;
            if (new_codeur21 != old_codeur21) {
            compteur_codeur21 = compteur_codeur21 + 1;
        }
        old_codeur21=new_codeur21;

        //printf("CompteurM3 = %d  \r\n", compteur_codeur31);
        //printf("CompteurM2 = %d  \r\n", compteur_codeur21);

    
    }
    moteur_2_avancer.write(0);
    moteur_2_reculer.write(0);
    moteur_3_avancer.write(0);
    moteur_3_reculer.write(0);    
    }
}



/* Main Function */
int main() {

    // data de distance et d'angle
    int distance=0;
    int angle=0;
    // compteur pour savoir si une nouvelle donnée est reçu
    int c=0; 
    int c_old=0;
    
    // init du module de communication radio
    initNRF24();

    //capteur themp hum
    int k = 0; //compteur
    float T, H; //température et humidité

    // PowerUp and initialize the module
    thread_sleep_for(WAIT_TIME_MS);    
    my_sensor.resetSensor();

    // init moteur
    int old_codeur31, new_codeur31;
    int old_codeur21, new_codeur21;
    int compteur_codeur31;
    int compteur_codeur21;

    moteur_2_avancer.period_ms(10);
    moteur_2_reculer.period_ms(10);
    moteur_3_avancer.period_ms(10);
    moteur_3_reculer.period_ms(10);  
    
    while(1) {
        // reception des données
        transmettreConsigne(&distance, &angle, &c);

        if (c>c_old) // si nouvelle données on les traites, sinon on écoute si de nouvelle données sont recu
        {
            c_old=c;

            // vitesse du moteur (rapport cyclique)
            double V=0.6;
            // executer moteur
            Tourner_angle(angle, V); // rotation
            wait_us(500000);
            Avancer_ou_reculer(distance, V); // distance

            // Collect T and H data from the sensor
            my_sensor.readTRH(&T, &H);
            //printf("[%4d] T=%f / H=%f \r\n\n ", k, T, H);
            //transmission des données
            transmettreTempHum(&T,&H);

        }
    }
}




