/****************************************************************************/
/*  nRF24L1 module library                                                  */
/****************************************************************************/
/*  LEnsE / Julien VILLEMEJANE       /   Institut d'Optique Graduate School */
/****************************************************************************/
/*  Library - nRF24.cpp file                                                */
/****************************************************************************/
/*  Tested on Nucleo-L476RG / 4th nov 2021                                  */
/****************************************************************************/

#include "nRF24.h"
#include <ctime>

#define TRANSFER_SIZE   8
#define TRANSFER_SIZE_CONSIGNE   8

nRF24L01P       nRF24_mod(PA_7, PA_6, PA_5, PA_12, PA_11, PB_12);
// MOSI, MISO, SCK, CSN, CE, IRQ
// Pin de connexions de l'antenne à ajuster au montage

char k;
char    dataToSend[TRANSFER_SIZE] = {0xAA, 0x01, 0x10, 0xF0,0xAA, 0x01, 0x10, 0xF0};
char    distanceAngle[TRANSFER_SIZE_CONSIGNE] = {50,           13,  84,   8,          5,   13,      84, 8};
//                                              signe_distance dist dist signe_angle dist dist      inutiles
char    dataReceived[TRANSFER_SIZE] = {0};
char    dataReceivedConsigne[TRANSFER_SIZE_CONSIGNE] = {0};
int     dataConvert[TRANSFER_SIZE_CONSIGNE] = {0};
char    rxDataCnt;


// Fonction d'initialisation du module BT nRF24L01
void initNRF24(){
    nRF24_mod.powerUp();
    wait_us(100000);
    nRF24_mod.setAirDataRate(NRF24L01P_DATARATE_250_KBPS);
    nRF24_mod.setRfFrequency(2400);
    wait_us(100000);
    debug_pc.printf( "nRF24L01+ Frequency    : %d MHz\r\n",  nRF24_mod.getRfFrequency() );
    debug_pc.printf( "nRF24L01+ Output power : %d dBm\r\n",  nRF24_mod.getRfOutputPower() );
    debug_pc.printf( "nRF24L01+ Data Rate    : %d kbps\r\n", nRF24_mod.getAirDataRate() );
    debug_pc.printf( "Transfers are grouped into %d characters\r\n", TRANSFER_SIZE );
    nRF24_mod.setTransferSize( TRANSFER_SIZE );
    nRF24_mod.setReceiveMode();
    nRF24_mod.enable();
}

// Fonction de test du module BT nRF24L01
void testNRF24(void){
    /* Lecture donnée depuis nRF24 */
    if ( nRF24_mod.readable() ) {

        // ...read the data into the receive buffer
        rxDataCnt = nRF24_mod.read( NRF24L01P_PIPE_P0, dataReceived, TRANSFER_SIZE);

        // Display the receive buffer contents via the host serial link
        debug_pc.printf("\tD = ");
        for ( int i = 0; i < rxDataCnt; i++ ) {
            debug_pc.printf(" %x \t", dataReceived[i]);
        }
        debug_pc.printf("\r\n");
    }   
}

// Fonction transmet distance et rotation du module BT nRF24L01
void transmettreConsigne(int *distance, int *angle, int *c){
    /* Lecture donnée depuis nRF24 */
    if ( nRF24_mod.readable() ) {

        // ...read the data into the receive buffer
        rxDataCnt = nRF24_mod.read( NRF24L01P_PIPE_P0, dataReceivedConsigne, TRANSFER_SIZE_CONSIGNE);

        // Convert the receive buffer contents via the host serial link
        // convertit le code ASCII en nombre
        for ( int i = 0; i < rxDataCnt; i++ ) {
            //debug_pc.printf(" %d \t", dataReceivedConsigne[i]);
            dataConvert[i] = dataReceivedConsigne[i] - 48;
        }
        // - correspond à -3
        // + correspond à -5

        *distance = dataConvert[1]*10 + dataConvert[2];
        if (dataConvert[0] == -3)
        {
            *distance = *distance * -1;
        }

        *angle = dataConvert[4]*10 + dataConvert[5];
        if (dataConvert[3] == -3)
        {
            *angle = *angle * -1;
        }
        
        debug_pc.printf("distance : %d \r\n angle : %d \r\n", *distance, *angle);
        //debug sur teraterm

        *c=*c + 1;
        //incr le compteur pour savoir si on a recu de nouvelle donnée ou pas
    }   
}

//faire fonction emission de temp et hum
void transmettreTempHum(float *T, float *H){

    char Tchar[8];
    int ret = snprintf(Tchar, sizeof Tchar, "%f", *T);
    // convertit le float en chaine de caractère

    
    //debug_pc.printf("%s \n", Tchar);

    Tchar[6]=' ';
    Tchar[7]='C'; //Afficher l'unité
    debug_pc.printf("%c%c%c%c%c%c%c%c \n", Tchar[0], Tchar[1], Tchar[2], Tchar[3], Tchar[4], Tchar[5], Tchar[6], Tchar[7]);
    // affiche pour le debug

    // envoi des données à l'autre carte
    nRF24_mod.setRfFrequency(2400);
    nRF24_mod.write( NRF24L01P_PIPE_P0, Tchar, 8 );
    debug_pc.printf( "SENT\r\n");
    wait_us(100000);

}
